@@ -453,7 +453,7 @@ TEST_F(PositiveSyncValWsi, ResyncWithSwapchain) {
     transition_swapchain_image0.image = swapchain_image0;
     transition_swapchain_image0.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
-    const SurfaceInformation info = GetSwapchainInfo(m_surface.Handle());
+    const SurfaceInformation info = GetSwapchainInfo(m_surface);
     const uint32_t width = info.surface_capabilities.minImageExtent.width;
     const uint32_t height = info.surface_capabilities.minImageExtent.height;
     const uint32_t format_size = vkuFormatTexelBlockSize(info.surface_formats[0].format);
@@ -566,7 +566,7 @@ TEST_F(PositiveSyncValWsi, ResyncWithSwapchain2) {
     transition_swapchain_image0.image = swapchain_image0;
     transition_swapchain_image0.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
-    const SurfaceInformation info = GetSwapchainInfo(m_surface.Handle());
+    const SurfaceInformation info = GetSwapchainInfo(m_surface);
     const uint32_t width = info.surface_capabilities.minImageExtent.width;
     const uint32_t height = info.surface_capabilities.minImageExtent.height;
     const uint32_t format_size = vkuFormatTexelBlockSize(info.surface_formats[0].format);
@@ -654,7 +654,7 @@ TEST_F(PositiveSyncValWsi, ResyncWithSwapchain3) {
     transition_swapchain_image0.image = swapchain_image0;
     transition_swapchain_image0.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
-    const SurfaceInformation info = GetSwapchainInfo(m_surface.Handle());
+    const SurfaceInformation info = GetSwapchainInfo(m_surface);
     const uint32_t width = info.surface_capabilities.minImageExtent.width;
     const uint32_t height = info.surface_capabilities.minImageExtent.height;
     const uint32_t format_size = vkuFormatTexelBlockSize(info.surface_formats[0].format);
@@ -716,13 +716,13 @@ TEST_F(PositiveSyncValWsi, ResyncWithSwapchain4) {
     RETURN_IF_SKIP(InitSyncVal());
     RETURN_IF_SKIP(InitSurface());
 
-    const SurfaceInformation surface_info = GetSwapchainInfo(m_surface.Handle());
+    const SurfaceInformation surface_info = GetSwapchainInfo(m_surface);
     if (surface_info.surface_capabilities.minImageCount > 3 || surface_info.surface_capabilities.maxImageCount < 3) {
         GTEST_SKIP() << "Surface must support swapchains with 3 images";
     }
 
     VkSwapchainCreateInfoKHR swapchain_ci = GetDefaultSwapchainCreateInfo(
-        m_surface.Handle(), surface_info, VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT);
+        m_surface, surface_info, VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT);
     swapchain_ci.minImageCount = 3;
     vkt::Swapchain swapchain(*m_device, swapchain_ci);
 
